/****************************************************************************
Author: Lucas Vinicius dos Santos Assis
 ****************************************************************************/

#ifndef _IMAGEMANAGER_H_
#define _IMAGEMANAGER_H_

// VisualCastalia trace structure
#include "ImageTransmissionTraces.h"

#include <vector> // Vector functions and string type
#include <stack>  // Stack functions and string type
#include <string> // String functions and string type

#include <sstream> // COnvert number to string
#include <iostream>
#include <fstream> // It allows us to save traces in files
#include <opencv2/opencv.hpp> // Opencv
#include <stdlib.h>     /* srand, rand */

using namespace std;
using namespace cv;

enum imageCompressionModes {
	JPG_MODE = 1000,
    PNG_MODE = 2000,
    BITMAP_MODE = 3000
};

class ImageManager {

	private:
		// Variables
		vector<unsigned char> localBuffer;
		stack<unsigned char> localBufferStack;
		
		// It performs VisualCastalia traces
		ImageTransmissionTraces visualCastaliaTrace;

		// Function
		vector<unsigned char> imageEncoder(string imageName, string imageFormat);
		void imageDecoder(vector<unsigned char> buffer, string path_to_save, int source);

	public:
		// Number of the node that contains this imageManager
		int numNode;
		// The simulation time that some function of this imageManager is called
		double simulationTime;

		// It puts the image content into a buffer
		int imgBufferPush(string imageName, string imageFormat);
		// This function returns an empty vector<uchar> if the buffer stack is already empty.
		// It returns imgPktSize cell from the image buffer array.
		vector<unsigned char> imgBufferPop(int imgPktSize);
		// Transform a valid complete buffer into an image file.
		// If it receives an image buffer with valid data, saves it
		// into an image file inside the path_to_save directory
		void storeRcvImage (vector<unsigned char> buffer, string path_to_save, int source);

		void generateRandomImages(int imgHeight,
									int imgWidth,
									bool useColoredImages, 
									string path_to_save,
									int caseToDraw);
		

};

#endif		// _IMAGEMANAGER_H_
