// Author: Lucas Vinicius dos Santos Assis
// Universidade Estadual de Feira de Santana

#include "SinkNodeManager.h"

// When a duplicated packet is received returns false
bool SinkNodeManager::receiveImagePacket (unsigned short nodeID,
											double locX,
											double locY,
											vector<unsigned char> bufferFragment,
											int imgPckNumber,
											int totalImagePkts,
											int setNumber){

	string key = getImgBufferKey (nodeID, setNumber, imgPckNumber);

	set<string>::iterator it;
	// Looking for the packet key in data base to avoid accept duplicated packets
	imgPckKeys.find(key);
	// Only store a new packet if it is not duplicated from data base
	if( it != imgPckKeys.end() ){
		visualCastaliaTrace.writeTrace( numNode, simulationTime,
					 "Received ImgPck - Key: " + key);
		imagePacketBuffer.insert(pair< string, vector<unsigned char> >(key, bufferFragment));
		imgPckKeys.insert (key);
	}
	else{
		visualCastaliaTrace.writeTrace( numNode, simulationTime,
					 "Received repeated ImgPck - Key: " + key);
		return false;
	}
	return true;
	
}

bool SinkNodeManager::isLastPacket (int totalImgPackets, int imagePktNumber, bool isNotDuplicated){
	// Definition of the last imgPacket
	if((imagePktNumber == totalImgPackets-1) && isNotDuplicated){
		return true;
	}
	return false;
}
// From sink node data base, gets all the fragmented data of a single image and groups it in "buffer"
vector<unsigned char> SinkNodeManager::getCompleteBuffer(unsigned short nodeID,
												int setNumber, 
												int bufferSize, 
												int totalImagePkts){
	vector<unsigned char> buffer;
	map<string, vector <unsigned char> >::iterator it;
	string key;
	for (int i = 0; i < totalImagePkts; i++){
		key = getImgBufferKey (nodeID, setNumber, i);
		it = imagePacketBuffer.find(key);
		if( it != imagePacketBuffer.end() ){
			visualCastaliaTrace.writeTrace( numNode, simulationTime,
					 "ImgPck - Key: " + key +" found. Grouping into complete buffer");
			buffer.insert(buffer.end(),
			imagePacketBuffer.find(key)->second.begin(),
			imagePacketBuffer.find(key)->second.end());
			
			// Erasing the data from data base to return it for image transmission app
			imagePacketBuffer.erase(key);
			visualCastaliaTrace.writeTrace( numNode, simulationTime,
					 "ImgPck - Key: " + key +" erased from sink data base");
		}
		else{
			visualCastaliaTrace.writeTrace( numNode, simulationTime,
					 "ImgPck - Key: " + key +" not found. Filling the space with zeros");
			vector<unsigned char> buffer_aux = vector<unsigned char>(bufferSize, '0');
			buffer.insert(buffer.end(),
						 buffer_aux.begin(), 
						 buffer_aux.end());
		}
	}
	return buffer;
}

string  SinkNodeManager::getImgBufferKey (unsigned short nodeID, int setNumber, int imgPckNumber){
	int idNode = int(nodeID);
	// key = nodeID-setNumber-imgPckNumber;
	string imgBufferKey;
	ostringstream convert, convert1, convert2;
	convert << idNode;
	imgBufferKey = convert.str();
	imgBufferKey += "-";
	
	convert1 << setNumber;
	imgBufferKey += convert1.str();
	imgBufferKey += "-";
	
	convert2 << imgPckNumber;
	imgBufferKey += convert2.str();
	return imgBufferKey;
}