#ifndef _SINKNODEMANAGER_H_
#define _SINKNODEMANAGER_H_

// VisualCastalia trace structure
#include "ImageTransmissionTraces.h"

// Data Structures
#include <map>
#include <vector>
#include <set>

#include <utility>      // std::pair, std::make_pair
#include <string>
#include <sstream>
#include <fstream> // It allows us to save traces in files
#include <iostream>
//#include <opencv2/opencv.hpp>

using namespace std;

class SinkNodeManager {
	public:
		// When the first image packet from a new node arrives in the sink node
		// the number of correspondent node in the vector turns 1. This allows
		// the sink node creates a corresponder folder to store the received images
		// from that sender node
		vector<bool> nodeIDList = vector<bool>(5000, false);

		// Number of the node whose packet was received
		int numNode;
		// The simulation time that some function of this sinkNodeManager is called
		double simulationTime;

		// When a duplicated packet is received returns false
		bool receiveImagePacket (unsigned short nodeID,
									double locX,
									double locY,
									vector<unsigned char> bufferFragment,
									int imgPckNumber,
									int totalImagePkts,
									int setNumber);

		bool isLastPacket ( int totalImgPackets, int imagePktNumber, bool isNotDuplicated);

		vector<unsigned char> getCompleteBuffer(unsigned short nodeID,
										int setNumber, 
										int pckSize, 
										int totalImagePkts);
	private:
		string getImgBufferKey (unsigned short nodeID, int setNumber, int imgPckNumber);
		
		map< string, vector<unsigned char> > imagePacketBuffer;

		// It performs VisualCastalia traces
		ImageTransmissionTraces visualCastaliaTrace;

		// Store the keys of image packet, avoid receivement of duplicated packets
		set<string> imgPckKeys;
};

#endif

